/*Test code for using the dual-Stepper board based on MCP23S17SS IO expander chip
and DRV8880 dual stepper driver chips 
*/
// inslude the SPI library:
#include <SPI.h>

#define    IODIRA    (0x00)      // MCP23x17 I/O Direction Register
#define    IODIRB    (0x01)      // 1 = Input (default), 0 = Output
#define    IOCON     (0x0A)      // MCP23x17 Configuration Register
#define    GPIOA     (0x12)      // MCP23x17 GPIO Port Register
#define    GPIOB     (0x13)      // Value on the Port - Writing Sets Bits in the Output Latch
#define    ADDR_ENABLE   (0b00001000) // Configuration register for MCP23S17
#define    StepperAddr   (0b01000000) 
#define    GPPUA     (0x06)     //Pull up resistor A
#define    GPPUB     (0x16)     //Pull up resistor B

// set pin 10 as the slave select;
const int ssPin = 10;

uint8_t IODIR_A = 0x01; //stepper data byte Bank A
uint8_t GPPU_A = 0x01; //set input pull up resistors
// nFault = bit 7; = input (pull up)
// ENABLE = bit 6; = output
// DIR = bit 5; = output
// STEP = bit 4; = output
// M1 = bit 3; = output
// M0 = bit 2; = input/output
// TRQ1 = bit 1; = output
// TRQ0 = bit 0; = output

uint8_t IODIR_B = 0x02; //stepper data byte Bank A
uint8_t GPPU_B = 0x02; //set input pull up resistors
// nSleep = bit 7; = output
// nFault = bit 6; = input (pull up)
// DIR = bit 5; = output
// STEP = bit 4; = output
// M1 = bit 3; = output
// M0 = bit 2; = input/output
// TRQ1 = bit 1; = output
// TRQ0 = bit 0; = output

uint8_t GPIO_A = 0; //stepper data byte Bank A
uint8_t GPIO_B = 0; //stepper data byte Bank B





  // set up the speed, data order and data mode
SPISettings settingsA(5000000, MSBFIRST, SPI_MODE0);

void stepA(){ //send setp pulse train _n_ 
  bitWrite(GPIO_A,4,0); //ensure step is back in 0 value
  digitalSPIWrite(StepperAddr, GPIOA, GPIO_A); //set GPIO_A
  bitWrite(GPIO_A,4,1); //ensure step is back in 1 value
  digitalSPIWrite(StepperAddr, GPIOA, GPIO_A); //set GPIO_A
  delay(100);
  bitWrite(GPIO_A,4,0); //ensure step is back in 0 value
  digitalSPIWrite(StepperAddr, GPIOA, GPIO_A); //set GPIO_A
}

void stepB(){ //send setp pulse train _n_ 
  bitWrite(GPIO_B,4,0); //ensure step is back in 0 value
  digitalSPIWrite(StepperAddr, GPIOB, GPIO_B); //set GPIO_B
  bitWrite(GPIO_B,4,1); //ensure step is back in 1 value
  digitalSPIWrite(StepperAddr, GPIOB, GPIO_B); //set GPIO_B
  delay(100);
  bitWrite(GPIO_B,4,0); //ensure step is back in 0 value
  digitalSPIWrite(StepperAddr, GPIOB, GPIO_B); //set GPIO_B
}


void setEnable(uint8_t EN){ //enable is called less often than flush, so pullup set here 
  bitWrite(GPIO_A,6,EN);
  
  bitWrite(GPIO_A,4,0); //ensure step is 0 value
  bitWrite(GPIO_B,4,0); //ensure step is 0 value

  //then apply changes
  digitalSPIWrite(StepperAddr, GPIOA, GPIO_A); //set GPIO_A
  digitalSPIWrite(StepperAddr, GPIOB, GPIO_B); //set GPIO_B
  digitalSPIWrite(StepperAddr, GPPUA, GPPU_A); //set GPPU_A (pull up resistors)
  digitalSPIWrite(StepperAddr, GPPUB, GPPU_B); //set GPPU_B (pull up resistors)
}


void setDirA(uint8_t DIR){
  bitWrite(GPIO_A,5,DIR); //DIR = 0/1
}

void setDirB(uint8_t DIR){
  bitWrite(GPIO_B,5,DIR); //DIR = 0/1
}


void setTorqueA(uint8_t TRQ){
  if (TRQ > 0 && TRQ <= 38){
    bitWrite(GPIO_A,0,1); //TRQ0 = 1
    bitWrite(GPIO_A,1,1); //TRQ1 = 1
  }

  if (TRQ > 38 && TRQ <= 63){
    bitWrite(GPIO_A,0,0); //TRQ0 = 0
    bitWrite(GPIO_A,1,1); //TRQ1 = 1
  }  

  if (TRQ > 63 && TRQ <= 88){
    bitWrite(GPIO_A,0,1); //TRQ0 = 1
    bitWrite(GPIO_A,1,0); //TRQ1 = 0
  }  

  if (TRQ > 88 && TRQ <= 255){
    bitWrite(GPIO_A,0,0); //TRQ0 = 0
    bitWrite(GPIO_A,1,0); //TRQ1 = 0
  }
  
}

void setTorqueB(uint8_t TRQ){
  if (TRQ > 0 && TRQ <= 38){
    bitWrite(GPIO_B,0,1); //TRQ0 = 1
    bitWrite(GPIO_B,1,1); //TRQ1 = 1
  }

  if (TRQ > 38 && TRQ <= 63){
    bitWrite(GPIO_B,0,0); //TRQ0 = 0
    bitWrite(GPIO_B,1,1); //TRQ1 = 1
  }  

  if (TRQ > 63 && TRQ <= 88){
    bitWrite(GPIO_B,0,1); //TRQ0 = 1
    bitWrite(GPIO_B,1,0); //TRQ1 = 0
  }  

  if (TRQ > 88 && TRQ <= 255){
    bitWrite(GPIO_B,0,0); //TRQ0 = 0
    bitWrite(GPIO_B,1,0); //TRQ1 = 0
  }
  
}

void setStepSizeA(uint8_t sSize){ //motor A or B
  if (sSize == 1){
      bitWrite(GPIO_A,2,0); //M0 = 0
      bitWrite(GPIO_A,3,0); //M1 = 0
  }

    if (sSize == 2){
      bitWrite(GPIO_A,2,0); //M0 = 0
      bitWrite(GPIO_A,3,1); //M1 = 1
  }

    if (sSize == 4){
      bitWrite(GPIO_A,2,1); //M0 = 1
      bitWrite(GPIO_A,3,1); //M1 = 1
  }

    if (sSize == 8){
      bitWrite(IODIR_A,2,1); //M0 = z (set to input)
      bitWrite(GPIO_A,3,0); //M1 = 0
  }

    if (sSize == 16){
      bitWrite(IODIR_A,2,1); //M0 = z (set to input)
      bitWrite(GPIO_A,3,1); //M1 = 1
  }


}


void setStepSizeB(uint8_t sSize){ //motor B
  if (sSize == 1){
      bitWrite(GPIO_B,2,0); //M0 = 0
      bitWrite(GPIO_B,3,0); //M1 = 0
  }

    if (sSize == 2){
      bitWrite(GPIO_B,2,0); //M0 = 0
      bitWrite(GPIO_B,3,1); //M1 = 1
  }

    if (sSize == 4){
      bitWrite(GPIO_B,2,1); //M0 = 1
      bitWrite(GPIO_B,3,1); //M1 = 1
  }

    if (sSize == 8){
      bitWrite(IODIR_B,2,1); //M0 = z (set to input)
      bitWrite(GPIO_B,3,0); //M1 = 0
  }

    if (sSize == 16){
      bitWrite(IODIR_B,2,1); //M0 = z (set to input)
      bitWrite(GPIO_B,3,1); //M1 = 1
  }

}


void flushGPIO(){
  //Flush updates through GPIO
  digitalSPIWrite(StepperAddr, IODIRA, IODIR_A); //set DIR_A
  digitalSPIWrite(StepperAddr, IODIRB, IODIR_B); //set DIR_B
  digitalSPIWrite(StepperAddr, GPIOA, GPIO_A); //set GPIO_A
  digitalSPIWrite(StepperAddr, GPIOB, GPIO_B); //set GPIO_B
}


void digitalSPIWrite(uint8_t addr, uint8_t reg, uint8_t data) {
  SPI.beginTransaction(settingsA);
  digitalWrite(ssPin, LOW); //take the SS pin low to select the chip:
  SPI.transfer(addr); //send chip address (fixed)
  SPI.transfer(reg); //send in the address and value via SPI:
  SPI.transfer(data);
  digitalWrite(ssPin, HIGH); // take the SS pin high to de-select the chip:
  SPI.endTransaction();
}


void setup() {
  //
  pinMode(ssPin, OUTPUT); //set the ssPin as an output:
  digitalWrite(ssPin, HIGH);
  SPI.begin(); //initialize SPI:
  digitalSPIWrite(StepperAddr, IOCON, ADDR_ENABLE);


  setDirA(0);
  setDirB(0);
  setTorqueA(75); //0 to 100%
  setTorqueB(75); //0 to 100%
  setStepSizeA(1); //1th, 2th, 4th, 8th, 16th
  setStepSizeB(1); //1th, 2th, 4th, 8th, 16th
  flushGPIO();

  
  setEnable(1); //enable
  bitWrite(GPIO_B,7,1); //nsleep high = awake
  flushGPIO();
}

void loop() {
  // put your main code here, to run repeatedly:
  stepA();
  //delay(1);
  stepB();
  //delay(1);
}



